unit editor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, common, NNTPComp, SMTPComp, ComCtrls, MyCheckBox,
  MyRichEdit, Menus, MyComboBox, ulinesock, MyListView, streams;

type
  TEditWindow = class(TForm)
    Headers: TPanel;
    bSend: TButton;
    StatusBar: TStatusBar;
    bStop: TButton;
    Label2: TLabel;
    Label1: TLabel;
    edName: TEdit;
    edPubEMail: TEdit;
    edEMail: TEdit;
    edOrg: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label9: TLabel;
    coIDs: TComboBox;
    bQueue: TButton;
    Label7: TLabel;
    coProfs: TComboBox;
    Pager: TPageControl;
    Tab1: TTabSheet;
    edGroups: TEdit;
    bEdGrps: TButton;
    Label10: TLabel;
    edFollowUp: TEdit;
    bEdFollowup: TButton;
    Label8: TLabel;
    edSubject: TEdit;
    cbXNA: TCheckBox;
    Tab2: TTabSheet;
    ScrollBox: TScrollBox;
    Tab3: TTabSheet;
    Label12: TLabel;
    bROT13: TButton;
    bGroups: TMyCheckBox;
    bMailTo: TMyCheckBox;
    bWrap: TMyCheckBox;
    bPretty: TButton;
    Memo: TMyRichEdit;
    Moreheaders: TMyRichEdit;
    bUndo: TButton;
    poMemo: TPopupMenu;
    miUndo: TMenuItem;
    N1: TMenuItem;
    miCut: TMenuItem;
    miCopy: TMenuItem;
    miPaste: TMenuItem;
    miDel: TMenuItem;
    N2: TMenuItem;
    miSelectAll: TMenuItem;
    miPasteQ: TMenuItem;
    miRewrap: TMenuItem;
    miRewrapJoin: TMenuItem;
    edMailTo: TMyComboBox;
    bAdd: TButton;
    Tab4: TTabSheet;
    Atts: TMyListView;
    bAAdd: TButton;
    bARemove: TButton;
    bAClear: TButton;
    OpenDialog: TOpenDialog;
    cbKeepCopy: TMyCheckBox;
    cbNewThread: TMyCheckBox;
    procedure bSendClick(Sender: TObject);
    procedure NNTPBanner(Sender: TObject);
    procedure NNTPBeginPost(Sender: TObject);
    procedure NNTPEndPost(Sender: TObject);
    procedure NNTPError( Sender: TObject; ErrCode:integer; ErrMsg: String);
    procedure SMTPError( Sender: TObject; ErrCode:integer;  ErrMsg: String);
    procedure SMTPOkay(Sender: TObject);
    procedure bStopClick(Sender: TObject);
    procedure MemoKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormCreate(Sender: TObject);
    procedure coIDsChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure bGroupsClick(Sender: TObject);
    procedure bMailToClick(Sender: TObject);
    procedure bQueueClick(Sender: TObject);
    procedure MemoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure bEdFollowupClick(Sender: TObject);
    procedure coProfsChange(Sender: TObject);
    procedure edGroupsDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure edGroupsDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure FormShow(Sender: TObject);
    procedure bWrapClick(Sender: TObject);
    procedure bROT13Click(Sender: TObject);
    procedure edPubEMailEnter(Sender: TObject);
    procedure MemoChange(Sender: TObject);
    procedure bPrettyClick(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure bUndoClick(Sender: TObject);
    procedure miPasteClick(Sender: TObject);
    procedure mipoMemoClick(Sender: TObject);
    procedure miPasteQClick(Sender: TObject);
    procedure poMemoPopup(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure bAddClick(Sender: TObject);
    procedure bAAddClick(Sender: TObject);
    procedure AttsChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure bAClearClick(Sender: TObject);
    procedure bARemoveClick(Sender: TObject);
    procedure edSubjectChange(Sender: TObject);

  private
    { Private declarations }
    NNTP: TNNTP;
    SMTP: TSMTP;
    Refs: String;
    Strm: TFileStream;
    Dirty,MailSent,NewsSent: boolean;
    ActiveArt: TArticle;
    // window to notify result of msg editing/sending
    // <>nil indicates editing an existing msg in out or sent
    NotifyWnd: TForm;
    ID: TID;

    procedure doPost;
    procedure AddSig;
    procedure SaveToMbx( B: integer);
    function Validate: boolean;
    procedure GetID;
    procedure GetProfiles;
    procedure SetStatusText( const S:string);
    procedure bCustomHeaderClick( Sender: TObject);
    property StatusText: string write SetStatusText;
    procedure SendData(Sender: TLineSock);
    procedure AddHeader( KeyWord, Val: string; Active:boolean);

  public
    Profile: TProfile;
    EditResults: TEditResults;

    procedure CreateParams( var Params: TCreateParams ); override;
    procedure Post( AProf: TProfile; const AGroups, AFollowup: string);
    procedure FollowUp( AProf: TProfile;
                        const AMsgID, AGroups, ASubj,
                        AAuth, AEmail, ARefs: string;
                        ADate: TDateTime;
                        byNews, byMail, byForw, Supersede,
                        XNA, StripSig: boolean;
                        Ptr: Pointer);
    procedure EditMsg( Wnd: TForm; Art: TArticle; Mem: TMemoryStream);
  end;

implementation

uses main, ufolder, articles, upicker, groups,
     uperlre, MIMEComp, clipbrd, uwrap, ubag;

{$R *.DFM}

// P is start of Q
function StrLeft( P, Q:PChar):boolean;
begin
  while (P^<>#0) and (Q^<>#0) do
    if P^<>Q^ then break else begin Inc(P); Inc(Q); end;
  Result:=P^=#0;
end;

procedure TEditWindow.CreateParams( var Params: TCreateParams );
begin
  Inherited CreateParams(Params);
  Params.WndParent:=GetDeskTopWindow;
// {$IFDEF release}
  MetricsBag.GetFormParams( 'editor', Self, Params, NewsFrame.MDIChildCount<=1);
// {$ENDIF}
end;

procedure TEditWindow.Post( AProf: TProfile; const AGroups,AFollowup: string);
begin
  Caption:='Post to: '+AGroups;
  edGroups.Text:=AGroups;
  edFollowup.Text:=AFollowup;
  Profile:=AProf;
  GetProfiles; GetID;
  Refs:='';
  cbXNA.Checked:=AlwaysXNA;
  bGroups.Checked:=AGroups<>'';
  bGroupsClick( nil); bMailToClick( nil);
  AddSig;
  edSubject.SetFocus;
//  Memo.SelStart:=0; Memo.Perform( EM_SCROLLCARET, 0, 0);
  Memo.ClearUndo; bUndo.Enabled:=false;
  bPretty.Enabled:=false; Dirty:=false;
end;

procedure TEditWindow.GetID;
var i:integer; s:string;
begin
  i:=Pos( ',', edGroups.Text);
  if i>0 then s:=Copy( edGroups.Text, 1, i-1) else s:=edGroups.Text;
  NewsFrame.GetID( Profile, s, ID);
  coIDs.Items.Clear;
  for i:=0 to IDs.Count-1 do
    coIDs.Items.Add( IDs[i]);
  coIDs.ItemIndex:=IDs.IndexOf( ID.Title);
  edPubEMail.Text:=ID.PubEmail;
  edEmail.Text:=ID.Email;
  edName.Text:=ID.Name;
  edOrg.Text:=ID.Org;
end;

procedure TEditWindow.GetProfiles;
var i:integer;
begin
  for i:=0 to Profiles.Count-1 do
    coProfs.Items.Add( Profiles[i]);
  coProfs.ItemIndex:=Profiles.IndexOf( Profile.Name);
end;

procedure TEditWindow.FollowUp( AProf: TProfile; const AMsgID, AGroups, ASubj,
      AAuth, AEmail, ARefs: string; ADate: TDateTime;
      byNews, byMail, byForw, Supersede, XNA, StripSig: boolean; Ptr: Pointer);
var P:PChar; Mem: TMemoryStream; c:char; n:integer;
    AGroup:string;

  procedure doWrite( const S:string);
  begin
    Mem.Write( S[1], Length( S));
  end;

begin
  if Supersede then AddHeader( 'Supersedes', AMsgID, true);
  edGroups.Text:=AGroups;
  Profile:=AProf; GetProfiles; GetID;
  Mem:=TMemoryStream.Create;
  Memo.SetFocus;
  cbXNA.Checked:=AlwaysXNA;// or XNA;
  if XNA then begin cbXNA.Color:=clYellow; cbXNA.Font.Color:=clBlack; end;
  if AGroups<>'' then
  begin
    n:=Pos( ',', AGroups);
    if n>0 then AGroup:=Copy( AGroups, 1, n-1)
    else AGroup:=AGroups;
  end else AGroup:='';
  if byForw then
  begin
    Caption:='Forwarding: '+ASubj;
    edSubject.Text:='FORW: '+ASubj;
    doWrite( MakeAttrib( ForwAttrib, AAuth, AEmail, AMsgID, AGroup, ADate));
    doWrite('== BEGIN forwarded message =='+CRLF+CRLF);
//    Q:='';
  end else
  begin
    if Supersede or (StrLIComp( PChar( ASubj), 'Re:', 3)=0) then
    begin
      edSubject.Text:=ASubj; //Q:='>';
    end else
    begin
      edSubject.Text:='Re: '+ASubj; //Q:='> ';
    end;
    edMailTo.Text:=CombineNameEmail( AAuth, AEmail);
    if (byNews and byMail) then
      doWrite('[posted and mailed]'+CRLF+CRLF);
    if Supersede then
    begin
      Caption:='Supersede: '+ASubj;
    end else
    if byNews then
    begin
      Caption:='FollowUp to: '+ASubj;
//      if XNA then
  //      doWrite( MakeAttrib( NewsAttrib, '???', '???@???', '<X-No-Archive-msg>', AGroup, ADate))
  //    else
        doWrite( MakeAttrib( NewsAttrib, AAuth, AEmail, AMsgID, AGroup, ADate));
    end else
    begin
      Caption:='Reply to: '+ASubj;
      doWrite( MakeAttrib( MailAttrib, AAuth, AEmail, AMsgID, AGroup, ADate));
    end;
  end;
  Refs:=ARefs;
  bGroups.Checked:=byNews; bMailTo.Checked:=byMail or byForw;
  bGroupsClick( nil); bMailToClick( nil);
  if byForw then
  begin
    bGroups.Enabled:=false;
    bWrap.Checked:=false; Memo.WordWrap:=false;
    Mem.Write( Ptr^, StrLen( Ptr));
    doWrite(CRLF+'== END forwarded message =='+CRLF+CRLF);
    edMailto.SetFocus;
  end else
  begin
    if bGroups.Checked and regex.Match( '.+,.+,.+', AGroups) then
      bEdFollowup.Click;
    P:=StrPos( Ptr, CRLF+CRLF);
    if P<>nil then
    begin
      Inc( P, 4);
      c:='>';
      while P^<>#0 do
      begin
        if StripSig and (StrLComp( P, '-- '+CRLF, 5)=0) then
          break else
        begin
          if not Supersede then Mem.Write( c, 1);//Q[1], Length(Q));
          while not (P^ in [#13,#10,#0]) do
          begin
            if P^<>#9 then Mem.Write( P^, 1) else Mem.Write( '     ', 5);
            Inc( P);
          end;
          if P^<>#0 then
          begin
            Inc( P);
            Mem.Write( #13#10, 2);
            if P^=#10 then Inc( P);
          end;
        end;
      end;
    end;
  end;
  c:=#0;
  Mem.Write( c, 1);
  Memo.SetTextBuf( Mem.Memory);
  Mem.Free;
  n:=Memo.GetTextLen;
  if StripSig or not Supersede then AddSig;
  if not byForw then
  begin
    Memo.SelStart:=n; Memo.Perform( EM_SCROLLCARET, 0, 0);
  end;
  Memo.ClearUndo; bUndo.Enabled:=false;
  Dirty:=byForw; bPretty.Enabled:=not byForw;
end;

procedure TEditWindow.SMTPOkay(Sender: TObject);
var s1, s2: string;
begin
  with SMTP do
  case State of
    msConnected:
      begin
        StatusText:='HELO';
        Helo;
      end;
    msHelo:
      begin
        StatusText:='MAIL FROM <'+edEMail.Text+'>';
        From:=edEmail.Text;
      end;
    msMail:
      begin
        ExtractNameEmail( edMailto.Text, s1, s2);
        StatusText:='RCPT TO <'+s2+'>';
        Rcpt:=s2;
      end;
    msRcpt:
      begin
        StatusText:='DATA';
        Send;
      end;
    msAskSend: SendData( SMTP);
    msSending:
      begin
        StatusText:='QUIT';
        Quit;
      end;
    else
      begin
        OnDisconnected:=nil; OnError:=nil; OnOkay:=nil;
        MailSent:=true;
        Include( EditResults, erMailSent);
        doPost;
      end;
  end;
end;

procedure TEditWindow.SMTPError( Sender: TObject; ErrCode:integer;  ErrMsg: String);
begin
  if SMTP<>nil then SMTP.Kill;
  ErrorMsg( ErrMsg);
  StatusText:='Error';
  Memo.Enabled:=true;
  bStop.Enabled:=false; bSend.Enabled:=true; bQueue.Enabled:=true;
end;

procedure TEditWindow.NNTPError( Sender: TObject; ErrCode:integer; ErrMsg: String);
begin
  if NNTP<>nil then begin NNTP.InUse:=false; NNTP.Kill; NNTP:=nil; end;
  ErrorMsg( ErrMsg);
  StatusText:='Error';
  Memo.Enabled:=true;
  bStop.Enabled:=false; bSend.Enabled:=true; bQueue.Enabled:=true;
end;

procedure TEditWindow.NNTPBanner(Sender: TObject);
begin
  StatusText:='Connected';
  NNTP.Post;
end;

procedure TEditWindow.NNTPBeginPost(Sender: TObject);
begin
  SendData( NNTP);
end;

procedure TEditWindow.SendData(Sender: TLineSock);
var i:integer; S:string;
begin
  with Sender do
  begin
    StatusText:='Sending...';
    if bGroups.Checked then
      Text:=WrapHeader( 'Newsgroups: ',edGroups.Text);
    Text:='Subject: '+edSubject.Text;
    if Sender=NNTP then
      Text:='From: '+edName.Text+' <'+edPubEMail.Text+'>'
    else
      Text:='From: '+edName.Text+' <'+edEMail.Text+'>';
    if (Sender=SMTP) and bMailTo.Checked then
      Text:='To: '+edMailTo.Text;
    if Refs<>'' then Text:=MakeRefs( Refs);
    if edOrg.Text<>'' then Text:='Organization: '+edOrg.Text;
    if ID.IDToken<>'' then Text:=MakeMsgID( ID.IDToken, Profile.Server);
    if cbXNA.Checked then Text:='X-No-Archive: yes';
{$IFNDEF release}
    if CompareText( Self.coIDs.Text, 'anon')<>0 then
{$ENDIF}
      Text:='User-Agent: '+XNewsReader;
    if bGroups.Checked and (edFollowUp.Text<>'') then
       Text:=WrapHeader( 'Followup-To: ',edFollowUp.Text);
    for i:=0 to ScrollBox.ComponentCount-1 do
    with TPanel( ScrollBox.Components[i]) do
      with TMyCheckBox( Components[1]) do
      if Checked then
      begin
        S:=Caption; with TEdit( Tag) do if Text<>'' then Sender.Text:=S+': '+Text;
      end;
    with MoreHeaders do
      for i:=0 to Lines.Count-1 do Sender.Text:=Lines[i];
    Text:='';
    for i:=0 to Memo.Lines.Count-1 do
    begin
      if (Memo.Lines[i]<>'') and (Memo.Lines[i][1]='.') then
         Text:= '.'+Memo.Lines[i]
      else Text:=Memo.Lines[i];
    end;
{$IFNDEF release}
    for i:=0 to Atts.Items.Count-1 do
    begin
      Strm.Free; Strm:=nil;
      try
        Strm:=TFileStream.Create( Atts.Items[i].Caption, fmOpenRead or fmShareDenyWrite);
        try
          Text:='begin 644 '+ExtractFileName(Atts.Items[i].Caption);
          while Strm.Position<Strm.Size do
          begin
            UUEncodeLine( Strm, S);
            if (S<>'') and (S[1]='.') then
               Text:= '.'+S
            else Text:=S;
          end;
          Text:='`'; Text:='end';
        finally
          Strm.Free; Strm:=nil;
        end;
      except
        On E:Exception do ErrorMsg( E.Message);
      end;
    end;
{$ENDIF}
    Text:='.';
    StatusText:='Done.  Waiting for confirmation...';
  end;
end;

procedure TEditWindow.NNTPEndPost(Sender: TObject);
begin
  MessageBeep( MB_OK);
  NewsSent:=true;
  Include( EditResults, erNewsSent);
  Close;
end;

procedure TEditWindow.SaveToMbx( B: integer);
var Mem: TMemoryStream; Art: TArticle;
    Mbx: TFolder; S:string;
    i:integer;

  procedure doWrite( const S: string);
  begin
    Mem.Write( S[1], Length( S));
  end;

begin
  Mem:=TMemoryStream.Create;
  Art:=TArticle.Create;
  if B=folOut then
    Mbx:=TFolder.Create( TFolderInfo( Folders.Objects[B]).FName, mtOut)
  else
    Mbx:=TFolder.Create( TFolderInfo( Folders.Objects[B]).FName, mtSent);
  Try
    doWrite( 'Subject: '+edSubject.Text+CRLF);
    if Refs<>'' then
      doWrite( 'References: '+Refs+CRLF);
    doWrite( 'Date: '+FormatDateTime( 'ddd, dd mmm yyyy hh:nn:ss ', Now-TZ/2400)
              +'GMT'+CRLF);
    doWrite( 'Lines: '+IntToStr( Memo.Lines.Count)+CRLF);
    if cbXNA.Checked then doWrite( 'X-No-Archive: yes'+CRLF);
    with Art do
    begin
      MsgID:=coIDs.Text+#9+edName.Text+#9+edEmail.Text+#9+
             edPubEmail.Text+#9+edOrg.Text+#9+ID.IDToken+#9+ID.SigFile;
      Subj:=edSubject.Text;
      if edMailto.Text<>'' then
      begin
        ExtractNameEmail( edMailTo.Text, Auth, Email);
        doWrite( 'To: '+edMailto.Text+CRLF);
      end;
      if edGroups.Text<>'' then
      begin
        Refs:=Self.Refs;
        Group:=Profile.Name+':'+edGroups.Text;
        doWrite( 'Newsgroups: '+Profile.Name+':'+edGroups.Text+CRLF);
        if edFollowUp.Text<>'' then
        begin
          Group:=Group+'!'+edFollowUp.Text;
          doWrite( 'Followup-To: '+edFollowup.Text+CRLF);
        end;
      end;
      doWrite( 'X-ID: '+MsgID+CRLF);
      fToiDateTime( Now, iDate, iTime);
      SetState( nsByMail, bMailto.Checked); SetState( nsMailSent, MailSent);
      SetState( nsByNews, bGroups.Checked); SetState( nsNewsSent, NewsSent);
      S:=IntToStr(
         Integer( bMailTo.Checked) or (Integer( GetState( nsMailSent)) shl 1) or
         (Integer( bGroups.Checked) shl 2) or (Integer( GetState( nsNewsSent)) shl 3));
      doWrite( 'X-Status: '+S+CRLF);
    end;
    for i:=0 to ScrollBox.ComponentCount-1 do
    with TPanel( ScrollBox.Components[i]) do
      with TMyCheckBox( Components[1]) do
      if Checked then
      begin
        S:=Caption;
        with TEdit( Tag) do if Text<>'' then doWrite( S+': '+Text+CRLF);
      end;
    with MoreHeaders do
      for i:=0 to Lines.Count-1 do doWrite( Lines[i]+CRLF);
    doWrite( CRLF);
    with Memo do
      for i:=0 to Lines.Count-1 do doWrite( Lines[i]+CRLF);
    Mem.Position:=0;
    Art.NumLines:=Memo.Lines.Count;
    Mbx.Archive( Art, Mem, Mem.Size, Art.Group);
    Dirty:=false;
    if B=folOut then Include( EditResults, erSavedOut)
    else Include( EditResults, erSavedSent);
  finally
    Mem.Free; Art.Free; Mbx.Free;
  end;
  if NotifyWnd=nil then
    NewsFrame.FolderChange( TFolderInfo( Folders.Objects[B]));
end;

function TEditWindow.Validate: boolean;
var i,Qcount:integer; s1, s2: string; lastLineQuoted,WarnWrap:boolean;
begin
  Result:=false;
  for i:=0 to ComponentCount-1 do
    if Components[i] is TEdit then
      with TEdit( Components[i]) do Text:=Trim(Text);
  if not Dirty or (Memo.Lines.Count=0) then
  begin
    ErrorMsg( 'You did not change anything');  exit;
  end;
  if Trim(edSubject.Text)='' then
  begin
    ErrorMsg( 'Subject is empty'); exit;
  end;
  if not bGroups.Checked and not bMailto.Checked then
  begin
    ErrorMsg( 'You need to either post to newgroup(s) or reply by mail');
    exit;
  end;
  if bMailto.Checked then
  begin
    ExtractNameEmail( edMailto.Text, s1, s2);
    if Pos( ',', s2)>0 then
    begin
      ErrorMsg( 'Sorry, this program cannot mail to multiple aaddresses');
      exit;
    end;
    if (s2='') or not SaneEmail( s2, false) then
    begin
      ErrorMsg( 'Missing or invalid To email'); exit;
    end;
    {
    if not SaneEmail( s2, false) then
    begin
      case MessageDlg( '<'+s2+'> does not look like a valid e-mail address.'
        +CRLF+CRLF+'Do you want to fix it?', mtWarning, mbyesnocancel, 0) of
        mrYes: begin edMailto.SetFocus; exit; end;
        mrCancel: exit;
      end;
    end;
    }
    if (edEmail.Text='') or not SaneEmail( edEmail.Text, false) then
    begin
      ErrorMsg( 'Missing or invalid private email'); exit;
    end;
    {
    if not SaneEmail( edEmail.Text, false) then
    begin
      case MessageDlg( '<'+edEmail.Text+'> does not look like a valid e-mail address.'
        +CRLF+CRLF+'Do you want to fix it?', mtWarning, mbyesnocancel, 0) of
        mrYes: begin edEMail.SetFocus; exit; end;
        mrCancel: exit;
      end;
    end;
    }
  end;
  if bGroups.Checked then
  begin
    if (edGroups.Text='') then
    begin
      ErrorMsg( 'Missing newsgroups');
      exit;
    end;
    if (edPubEmail.Text='') or not SaneEmail( edPubEmail.Text, true) then
    begin
      ErrorMsg( 'Missing or invalid public email'); exit;
    end;
    {
    if not SaneEmail( edPubEmail.Text, true) then
    begin
      case MessageDlg( '<'+edPubEmail.Text+'> does not look like a valid e-mail address.'
        +CRLF+CRLF+'Do you want to fix it?', mtWarning, mbyesnocancel, 0) of
        mrYes: begin edPubEMail.SetFocus; exit; end;
        mrCancel: exit;
      end;
    end;
    }
  end;
  if bPretty.Enabled and (WarnBadWrap or (WarnExcessiveQuote>0)) then
  begin
    Qcount:=0; WarnWrap:=WarnBadWrap; lastLineQuoted:=false;
    with Memo do
    for i:=0 to Lines.Count-1 do
    begin
      s1:=Lines[i];
      if (s1<>'') and (s1[1]='>') then
      begin
        Inc( Qcount); //regex.Match( s1) then
        lastLineQuoted:=true;
      end else
      begin
        if WarnWrap and lastlineQuoted and (TrimLeft(s1)<>'') and (Length(s1)<20) then
          case MessageDlg('There seems to be badly wrapped quoted text'+#13+#10+
                          'Do you want it fixed?',
                          mtConfirmation, mbYesNoCancel, 0) of
            mrCancel: exit;
            mrYes: begin bPretty.Click; exit; end;
            else WarnWrap:=false;
          end;
        lastLineQuoted:=false;
      end;
    end;
    if (WarnExcessiveQuote>0) and
       (Muldiv( Qcount, 100, Memo.Lines.Count)>WarnExcessiveQuote) and
       (MessageDlg('Your quoted text exceeds '+IntToStr(WarnExcessiveQuote)+'% of total lines'+#13+#10+
        'Do you want to trim quoted text?',
        mtWarning, mbYesNoCancel, 0)<>mrNo) then exit;
  end;

  with MoreHeaders do
    for i:=Lines.Count-1 downto 0 do
    begin
      Lines[i]:=Trim( Lines[i]);
      if Lines[i]='' then begin Lines.Delete( i); continue; end;
      with CustomHeaderRE do
        if Match( Lines[i]) and (SubExpCount>=5) and (SubExp[1].Text='') then
        begin
          if (CustomHeaders.IndexOf( SubExp[3].Text)>=0) or
             (Pos( '|'+LowerCase( SubExp[3].Text)+'|', ReservedHeaders)<>0) then
          begin
            ErrorMsg( 'Unsupported, duplicate or reserved manual header: '+Lines[i]); exit;
          end;
        end else
        begin
          ErrorMsg( 'Invalid manual header: '+Lines[i]); exit;
        end;
    end;
  if cbNewThread.Checked then Refs:='';
  Result:=true;
end;

procedure TEditWindow.bSendClick(Sender: TObject);
begin
  if not Validate then exit;
  Memo.Enabled:=false;
  bStop.Enabled:=true; bSend.Enabled:=false;  bQueue.Enabled:=false;
  if MailSent or not bMailTo.Checked then
  begin
    doPost;
    exit;
  end;
  if SMTP<>nil then SMTP.Free;
  SMTP:=TSMTP.Create;
  with SMTP do
  begin
    OnError:=SMTPError;
    OnOkay:=SMTPOkay;
    OnDisconnected:=bStopClick;
    StatusText:='Connecting to SMTP host...';
    Host:=Profile.SMTPHost;
    Open;
  end;
end;

procedure TEditWindow.doPost;
begin
  if not bGroups.Checked then
  begin
    Close;
    exit;
  end;
  if NNTP=nil then NNTP:=NewsFrame.AllocSocket( Profile);
  with NNTP do
  begin
    OnBanner:=NNTPBanner;
    OnBeginPost:=NNTPBeginPost;
    OnEndPost:=NNTPEndPost;
    OnError:=NNTPError;
    OnDisconnected:=bStopClick;
    if Connected then Post else
    begin
      StatusText := 'Connecting to '+Profile.Server;
      Open;
    end;
  end;
end;

procedure TEditWindow.bStopClick(Sender: TObject);
begin
  if NNTP<>nil then begin NNTP.InUse:=false; NNTP.Kill; NNTP:=nil; end;
  if SMTP<>nil then begin SMTP.Kill; end;
  Strm.Free; Strm:=nil;
  StatusText:='Stopped';
  Memo.Enabled:=true;
  bStop.Enabled:=false; bSend.Enabled:=true; bQueue.Enabled:=true;
end;

procedure TEditWindow.MemoKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var L,C:integer;
begin
  L:=SendMessage( Memo.Handle, EM_LINEFROMCHAR, -1, 0);
  C:=Memo.SelStart-SendMessage( Memo.Handle, EM_LINEINDEX, -1, 0);
  StatusBar.Panels[1].Text:=SysUtils.Format( 'Line %d  Col %d',
     [L+1,C+1]);
end;

procedure TEditWindow.FormCreate(Sender: TObject);
begin
  MailSent:=false; NewsSent:=false;
  SMTP:=nil;
  NNTP:=nil;
  ID:=TID.Create;
  NotifyWnd:=nil; ActiveArt:=nil; EditResults:=[];
  cbKeepCopy.Checked:=KeepCopies;
{$IFDEF release}
  Tab4.TabVisible:=false;
{$ENDIF}
end;

procedure TEditWindow.AddSig;
var F:TextFile; S: string; i,n:integer;
begin
  if ExtractFilePath( ID.SigFile)='' then ID.SigFile:=HomeDir+ID.SigFile;
  if not FileExists( ID.SigFile) then exit;
  AssignFile( F, ID.SigFile);
  Reset( F);
  with Memo do
  begin
    n:=GetTextLen; SelStart:=n;
    i:=0;
    while not EOF( F) do
    begin
      readln( F, S);
      // check for sig separator, even the false one
      if (n>=0) and ((S='--') or (S='-- ')) then
      begin
        Lines.Add( '-- ');
        n:=-1; i:=0;  // stuff above -- doesn't count
      end else
      begin
        Lines.Add( S);
        Inc( i); if i>10 then break;
      end;
    end;
    // insert sig separator if not found
    if n>=0 then
    begin
      SelStart:=n; SelText:=CRLF+'-- '+CRLF;
    end;
  end;
  CloseFile( F);
end;

procedure TEditWindow.coIDsChange(Sender: TObject);
var NewID: TID;
begin
  if coIDs.ItemIndex in [0..IDs.Count-1] then
  begin
    NewID:=TID( IDs.Objects[coIDs.ItemIndex]);
    with NewID do
    begin
      edPubEMail.Text:=PubEmail;
      edEmail.Text:=Email;
      edName.Text:=Name;
      edOrg.Text:=Org;
    end;
    ID.Assign( NewID);
  end;
end;

procedure TEditWindow.FormDestroy(Sender: TObject);
begin
  SMTP.Free; Strm.Free;
  if NNTP<>nil then NNTP.InUse:=false;
  ID.Free;
end;

procedure TEditWindow.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  CanClose:=true;
  if Dirty then
  begin
    if (MailSent or NewsSent) and
       (not bMailTo.Checked or MailSent) and
       (not bGroups.Checked or NewsSent) then
    begin
      if cbKeepCopy.Checked then SaveToMbx( folSent);
    end else
    case MessageDlg( 'Message not sent.  Do you want to save it?',
                     mtConfirmation, mbYesNoCancel, 0) of
      mrYes:
         begin
           if (NotifyWnd=nil) or
              (TArticlesWindow(NotifyWnd).FolderInfo.Kind=mtOut) then
              SaveToMbx( folOut)
           else SaveToMbx( folSent);
         end;
      mrCancel: CanClose:=false;
    end;
  end;
  if CanClose and ( NotifyWnd<>nil) then
    TArticlesWindow( NotifyWnd).EditMsgNotify( EditResults, ActiveArt);
// {$IFDEF release}
  //debugwrite( 'setformplace');
  MetricsBag.SetFormPlace( 'editor', Self, false);
// {$ENDIF}
end;

procedure TEditWindow.bGroupsClick(Sender: TObject);
begin
  edGroups.Enabled:=bGroups.Checked;
  if edGroups.Enabled then edGroups.Color:=clWindow
  else edGroups.Color:=clBtnFace;
  bSend.Enabled:=bGroups.Checked or bMailto.Checked;
  bQueue.Enabled:=bSend.Enabled;
end;

procedure TEditWindow.bMailToClick(Sender: TObject);
begin
  edMailto.Enabled:=bMailto.Checked;
  if edMailto.Enabled then edMailto.Color:=clWindow
  else edMailto.Color:=clBtnFace;
  bSend.Enabled:=bGroups.Checked or bMailto.Checked;
  bQueue.Enabled:=bSend.Enabled;
end;

procedure TEditWindow.EditMsg( Wnd: TForm; Art: TArticle; Mem: TMemoryStream);
var i,j:integer; P:PChar; FProfName, s: string; Mime:TMime; Headers: THeaders;
    CustHdrs:TStringList;dummy:boolean;
begin
  NotifyWnd:=Wnd; ActiveArt:=Art;
  Caption:='Edit message '+Art.Subj;
  edSubject.Text:=Art.Subj;
  edMailto.Text:=CombineNameEmail( Art.Auth, Art.Email);
  bMailto.Checked:=Art.GetState( nsByMail) and not Art.GetState( nsMailSent);
  bGroups.Checked:=Art.GetState( nsByNews) and not Art.GetState( nsNewsSent);
  P:=PChar( Art.Group);
  if StrPos( P, ':')=nil then edGroups.Text:=Art.Group else
  begin
    FProfName:=EatTokenX( P, [':']);
    edGroups.Text:=EatTokenX( P, ['!']);
    if P^<>#0 then edFollowUp.Text:=EatLine( P);
  end;
  i:=Profiles.IndexOf( FProfName);
  if i<0 then Profile:=TProfile( Profiles.Objects[0])
  else Profile:=TProfile( Profiles.Objects[i]);
  P:=PChar( Art.MsgID);
  Refs:=Art.Refs;
  ID.Title:=EatTokenX( P, [#9]);
  GetProfiles; GetID;
  ID.Name:=EatTokenX( P, [#9]); edName.Text:=ID.Name;
  ID.Email:=EatTokenX( P, [#9]); edEmail.Text:=ID.Email;
  ID.PubEmail:=EatTokenX( P, [#9]); edPubEmail.Text:=ID.PubEmail;
  ID.Org:=EatTokenX( P, [#9]); edOrg.Text:=ID.Org;
  ID.IDToken:=EatTokenX( P, [#9]);
  Memo.SetFocus;
  bGroupsClick( nil); bMailToClick( nil);
  P:=Mem.Memory;
  CustHdrs:=TStringList.Create;
  Mime:=TMime.Create; Mime.PHdrs:=@Headers; Mime.CustHdrs:=CustHdrs;
  Mime.BeginMsg;
  while P^<>#0 do
  begin
    Mime.ProcessLine( P, s, dummy, false, false, nil);
    if Mime.Notify=mnHeaderEnd then break;
  end;
  cbXNA.Checked:=Headers.XNoArchive;
  for i:=0 to ScrollBox.ComponentCount-1 do
    with TPanel( ScrollBox.Components[i]) do
    begin
      TMyCheckBox( Components[1]).Checked:=false;
//      TEdit( Components[0]).Enabled:=false;
//      TEdit( Components[0]).Color:=clBtnFace;
    end;
  with CustHdrs do
  for i:=0 to Count-1 do
  if Strings[i]='Supersedes' then
    AddHeader( 'Supersedes', THeader( Objects[i]).Val, true)
  else begin
    j:=CustomHeaders.IndexOf( Strings[i]);
    if (j>=0) and (j<ScrollBox.ComponentCount) then
    with TPanel( ScrollBox.Components[j]) do
    begin
      with TEdit( Components[0]) do
      begin
        Text:=THeader( CustHdrs.Objects[i]).Val;
        Enabled:=true; Color:=clWindow;
      end;
      TMyCheckBox( Components[1]).Checked:=true;
    end else
    with THeader( Objects[i]) do
    begin
      MoreHeaders.Lines.Add( KeyWord+': '+Val);
    end;
  end;
  FreeSList( CustHdrs); Mime.Free;
  P:=StrPos( Mem.Memory, CRLF+CRLF);
  if P<>nil then
  begin
    Inc( P, 4);
    Memo.SetTextBuf( P);
  end;
  Memo.SelStart:=0; Memo.Perform( EM_SCROLLCARET, 0, 0);
  Memo.ClearUndo; bUndo.Enabled:=false;
  Dirty:=true;
end;

procedure TEditWindow.bQueueClick(Sender: TObject);
begin
  if not Validate then exit;
  if (NotifyWnd=nil) or
     (TArticlesWindow(NotifyWnd).FolderInfo.Kind=mtOut) then
     SaveToMbx( folOut)
  else SaveToMbx( folSent);
  Close;
end;

procedure TEditWindow.MemoMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var K:Word;
begin
  MemoKeyUp(Sender, K, Shift);
end;

procedure TEditWindow.bEdFollowupClick(Sender: TObject);
begin
  if edGroups.Text='' then exit;
  with TfmGrpPicker.Create( Self) do
  Try
    if Sender=bEdFollowup then
    begin
      if ShowModalSelect( edGroups.Text, edFollowup.Text)=mrOk then
        edFollowup.Text:=Sel;
    end else
      if (ShowModalSelect( edGroups.Text)=mrOk) and (Sel<>'') then
        edGroups.Text:=Sel;
  Finally
    Free;
  end;
end;

procedure TEditWindow.coProfsChange(Sender: TObject);
begin
  if (coProfs.ItemIndex in [0..Profiles.Count-1]) then
  begin
    Profile:=TProfile( Profiles.Objects[coProfs.ItemIndex]);
    GetID;
  end;
end;

// accept groups dropped from group window
procedure TEditWindow.edGroupsDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var s:string;
    oldList,newList:TStringList; i:integer;
begin
  TGroupsWindow( TControl( Source).Owner).GetSelectedGroups( s);
  if s='' then exit;
  oldList:=TStringList.Create;
  newList:=TStringList.Create;
  Try
    // first split comma string to list
    with TPerlre.Create( false, 0) do
    Try
      Split( ',', TEdit( Sender).Text, oldList);
      Split( ',', s, newList);
    Finally
      Free;
    end;
    // merge list
    for i:=0 to newList.Count-1 do
      if oldList.IndexOf( newList[i])<0 then
        oldList.Add( newList[i]);
    if oldList.Count>MaxXPostAllowed then
      ErrorMsg( 'Sorry, you are not allowed to crosspost to more than '
        +IntToStr( MaxXPostAllowed)+' groups') else
    begin
      // join list back to comma string
      s:='';
      for i:=0 to oldList.Count-1 do
        if s<>'' then s:=s+','+oldList[i]
        else s:=oldList[i];
      TEdit( Sender).Text:=s;
      if (Sender=edGroups) and (oldList.Count>=3) then
      with TfmGrpPicker.Create( Self) do
      Try
        if ShowModalSelect( s, '')=mrOk then
          edFollowup.Text:=Sel;
      Finally
        Free;
      end;
    end;
  Finally
    oldList.Free; newList.Free;
  end;
end;

procedure TEditWindow.edGroupsDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:=(Source is TListView) and ((TControl( Source).Name='Grps') or
  (TControl( Source).Name='AllGrps'));
end;

const
  chars65 = '123456789012345678901234567890123456789012345678901234567890123456';

procedure TEditWindow.FormShow(Sender: TObject);
var i:integer; s:string;
begin
  with DisplayBag do
  begin
    Label1.Font.Assign( GetFont( 'EditorFont', Label1.Font));
    Label2.Font.Assign( Label1.Font);
    Label1.Font.Color:=clBtnText; Label2.Font.Color:=clBtnText;
    Memo.Font.Assign( Label1.Font);
    Memo.Color:=GetColor( 'ViewerColor', Memo.Color);
    Memo.Font.Color:=GetFont( 'ViewerFont', Memo.Font).Color;
    StatusBar.Font.Assign( GetFont( 'StatusFont', StatusBar.Font));
  end;
  s:='';
  for i:=66 to WrapColumn do s:=s+'0';
  i:=Label1.Canvas.TextWidth( chars65)+VScrollBarWidth;
  Constraints.MaxWidth:=i+Label1.Canvas.TextWidth( '678901234567890');
  Width:=i+Label1.Canvas.TextWidth( s);
  Constraints.MinWidth:=Width;
  StatusBar.Font.Color:=clBtnText;
  with CustomHeaders do
  for i:=0 to Count-1 do
  with THeader( Objects[i]) do
  begin
    AddHeader( KeyWord, Val, Active);
  end;
  ScrollBox.VertScrollBar.Range:=CustomHeaders.Count*26;
{$IFNDEF release}
//  Top:=0;
  //Height:=GetSystemMetrics( SM_CYMAXIMIZED)-4;
{$ENDIF}
end;

procedure TEditWindow.SetStatusText(const S: string);
begin
  StatusBar.Panels[0].Text:=S;  StatusBar.Update;
end;

procedure TEditWindow.bCustomHeaderClick(Sender: TObject);
begin
  with TMyCheckBox( Sender) do
    with TEdit( Tag) do
    if Checked then
    begin
      Enabled:=true; Color:=clWindow;
    end else
    begin
      Enabled:=false; Color:=clBtnFace;
    end;
end;

procedure TEditWindow.bWrapClick(Sender: TObject);
begin
  Memo.WordWrap:=bWrap.Checked;
end;

procedure TEditWindow.bROT13Click(Sender: TObject);
var ed:TCustomEdit; b:boolean;
begin
  b:=Dirty;
  if bROT13.Tag=0 then ed:=Memo else ed:=TCustomEdit( bROT13.Tag);
  with ed do
    if SelLength=0 then Text:=ROT13( Text)
    else SelText:=ROT13( SelText);
  Dirty:=b;
end;

procedure TEditWindow.edPubEMailEnter(Sender: TObject);
begin
  bROT13.Tag:=Integer( Sender);
end;

procedure TEditWindow.AddHeader(KeyWord, Val: string; Active: boolean);
var pn:TPanel; ed:TEdit;
begin
    pn:=TPanel.Create( ScrollBox);
    with pn do
    begin
      Parent:=ScrollBox; Align:=alTop; BevelOuter:=bvNone; Height:=26;
    end;
    ed:=TEdit.Create( pn);
    with ed do
    begin
      Parent:=pn; Left:=127; Top:=1; Width:=481; Height:=24; AutoSize:=False;
      Ctl3D:=False; ParentCtl3D:=false;
      Text:=Val;
    end;
    with TMyCheckBox.Create( pn) do
    begin
      Parent:=pn; Left:=4; Top:=1; Width:=109; Height:=22;
      Caption:=KeyWord; OnClick:=bCustomHeaderClick;
      HorAlignment:=haCenter; TabOrder:=0;
      LikeButton:=true; Tag:=Integer(ed); Checked:=Active;
      if not Active then
      begin
        ed.Enabled:=false; ed.Color:=clBtnFace;
      end;
    end;
end;

procedure TEditWindow.MemoChange(Sender: TObject);
begin
  if Memo.Modified then
  begin
    Dirty:=true; bUndo.Enabled:=Memo.canUndo;
  end;
end;

procedure TEditWindow.bPrettyClick(Sender: TObject);
var P:PChar; b:boolean;
begin
  b:=Dirty;
  with Memo do
  begin
    if SelLength=0 then Memo.SelectAll;
    P:=PChar( SelText);
    P:=RewrapText( @P, 0, StrLen( P), WrapColumn-3, Sender=miRewrapJoin);
    if P<>nil then
    begin
      if SelLength=0 then SetTextBuf( P) else SetSelTextBuf( P);
      FreeMem( P);
    end;
  end;
  Memo.SetFocus;
  Dirty:=b;
end;

procedure TEditWindow.MemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var s:string;
begin
  if (Shift=[ssShift]) and (Key=VK_INSERT) or
     (Shift=[ssCtrl]) and (Key=Word('V')) then
  begin
    miPasteClick(nil); Key:=0;
  end else
  // turn tabs into spaces
  if (Shift=[]) and (Key=VK_TAB) then
  begin
    Key:=0;
    s:='    ';
    Memo.Perform( EM_REPLACESEL, 0, Longint( @s[1]));
  end;
end;

procedure TEditWindow.bUndoClick(Sender: TObject);
begin
  Memo.Undo;  Memo.SetFocus;
end;

procedure TEditWindow.miPasteClick(Sender: TObject);
var s:string;
begin
  s:=ClipBoard.AsText+#0;
  Memo.Perform( EM_REPLACESEL, 0, Longint( @s[1]));
end;

procedure TEditWindow.mipoMemoClick(Sender: TObject);
begin
  with Memo do
    if Sender=miUndo then Undo
    else if Sender=miCut then CutToClipboard
    else if Sender=miCopy then CopyToClipboard
    else if Sender=miDel then ClearSelection
    else SelectAll;
end;

procedure TEditWindow.miPasteQClick(Sender: TObject);
var s:string; i:integer;
begin
{
  if StrLIComp( PChar( edSubject.Text), 'Re:', 3)=0 then
  begin
    Q:='>'; L:=1;
  end else
  begin
    Q:='> ';L:=2;
  end;
  }
  s:='>'+ClipBoard.AsText;
  i:=1;
  while i<=Length(s) do
    if s[i]=#10 then
    begin
      Inc( i); Insert( '>', s, i); Inc( i, 1);
    end else Inc( i);
  Memo.Perform( EM_REPLACESEL, 0, Longint( @s[1]));
end;

procedure TEditWindow.poMemoPopup(Sender: TObject);
begin
  with Memo do
  begin
    miUndo.Enabled:=CanUndo;
    miCut.Enabled:=SelLength>0;
    miCopy.Enabled:=miCut.Enabled;
    miDel.Enabled:=miCut.Enabled;
    miRewrap.Enabled:=bPretty.Enabled;
    miSelectAll.Enabled:=GetTextLen>0;
    miPaste.Enabled:=Clipboard.AsText<>'';
    miPasteQ.Enabled:=miPaste.Enabled;
  end;
end;

procedure TEditWindow.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action:=caFree;
end;

procedure TEditWindow.bAddClick(Sender: TObject);
begin
  if edMailto.Text<>'' then
  begin
    edMailto.AddText; MessageBeep($FFFFFFFF);
  end;
end;

procedure TEditWindow.bAAddClick(Sender: TObject);
var i,j:integer; b:boolean;
begin
  with OpenDialog,Atts do
  if Execute then
    for i:=0 to Files.Count-1 do
    begin
      b:=true;
      for j:=0 to Items.Count-1 do
        if CompareText(Items[j].Caption,Files[i])=0 then
        begin
          b:=false; break;
        end;
      if b then with Items.Add do Caption:=Files[i];
    end;
end;

procedure TEditWindow.AttsChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
begin
  bARemove.Enabled:=Atts.ItemFocused<>nil;
  bAClear.Enabled:=Atts.Items.Count>0;
end;

procedure TEditWindow.bAClearClick(Sender: TObject);
begin
  Atts.Items.Clear;
end;

procedure TEditWindow.bARemoveClick(Sender: TObject);
var i:integer;
begin
  with Atts do
    if ItemFocused<>nil then
    begin
      i:=ItemFocused.Index;
      Items.Delete( i);
      if i<Items.Count then VMFocused:=i else VMFocused:=Items.Count-1;
    end;
end;

procedure TEditWindow.edSubjectChange(Sender: TObject);
begin
  if (Refs<>'') then cbNewThread.Enabled:=true;
end;

end.























