{DEFINE DEBUG}

{ ********************************************************************************
 Name : Overseer Message Expert
 type : Delphi Expert

 Date                 Version       Programmer        Remarks
 03.03.1999 17:32:00  pre rel       nicolas foong     Create
 12.04.1999 08:41:02  pre rel       nicolas foong     - Optional generation of $IFDEF
                                                      - Hotkey for expert activation -> Add "ShortCut(Ord('D'), [ssCtrl, ssAlt])"
                                                      - Show expert form in screen center (not on designed position).
 Used by
 Delphree - Nexus - Overseer Project

 Description
 The Overseer Message Expert is a tool to make it easier to enter the
 Overseer statements in to your source code.

 REMARKS
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


 Copyright (c) by Nicolas Foong <nicolas.foong@pks.ch> CH_6300 Zug
******************************************************************************** }

unit uOverSeerExp01;

interface

uses
  Windows, Classes, ToolIntf, EditIntf, ExptIntf, VirtIntf, SysUtils, Menus, Forms,
  Dialogs, Controls;

type

  TOverseerExpert = class(TIExpert)
    menEditExp: TIMenuItemIntf; // Menu Items
    constructor Create; virtual;
    destructor Destroy; override;
    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    procedure menDbgClick(Sender: TIMenuItemIntf);
    procedure Execute; override;

  end;

procedure Register;

var
  expOverseer  : TOverseerExpert = nil;

implementation
uses uDbg, uDbgIntf, uOverSeerExpFrm;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}
{ Constructor and menu creation                                               }

constructor TOverseerExpert.Create;
var
  menMain      : TIMainMenuIntf;
  menToolsTools: TIMenuItemIntf;
  menToolsMenu : TIMenuItemIntf;
begin
  inherited Create;
  menEditExp := nil;
  if ToolServices <> nil then
  begin
    menMain := ToolServices.GetMainMenu;
    if menMain <> nil then
    begin
      try
        menToolsTools := menMain.FindMenuItem('ToolsOptionsItem');
        if menToolsTools <> nil then
        begin
          try
            menToolsMenu := menToolsTools.GetParent;
            if menToolsMenu <> nil then
            begin
              try
                menEditExp := menToolsMenu.InsertItem(menToolsTools.GetIndex + 1, 'Insert &Debug Message', 'Overseer1Item', '',
                  ShortCut(Ord('D'), [ssCtrl, ssAlt]), 0, 0, [mfEnabled, mfVisible], menDbgClick);
                if menEditExp = nil then
                ShowMessage('menEditExp = nil');

              finally
                menToolsMenu.DestroyMenuItem;
              end;
            end
            else
              ShowMessage('menToolsMenu <> nil');
          finally
            menToolsTools.DestroyMenuItem;
          end;
        end
        else
          ShowMessage('menToolsTools <> nil');
      finally
        menMain.Free;
      end;
    end
    else
      ShowMessage('menMain <> nil');
  end
  else
    ShowMessage('ToolServices <> nil');
end;


{* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *}

destructor TOverseerExpert.Destroy;
begin
  if menEditExp <> nil then menEditExp.DestroyMenuItem;
  inherited Destroy;
end;


procedure TOverseerExpert.menDbgClick(Sender: TIMenuItemIntf);

  function space(n: Integer): string;
  var
    i          : integer;
  begin
    if n > 50 then n := 50;
    Result := '';
    for i := 0 to n - 1 do
      Result := Result + ' ';
  end;

var
  EditIntf     : TIEditorInterface;
  ModIntf      : TIModuleInterface;
  EditWriter   : TIEditWriter;
  EditView     : TIEditView;
  EditPos      : TEditPos;
  CharPos      : TCharPos;
  fname        : string;
  i            : Integer;
  Dlg          : TfrmConfig;
  lPos         : Longint;
  Command      : string;

begin
  Dlg := TfrmConfig.Create(Application);
  Dlg.DebugString := '';
  Dlg.CompilerCond := '';
  try
    Dlg.Enabled := true;
    Dlg.Visible := false;
    Dlg.ShowModal;
    if Dlg.ModalResult = mrOK then
    begin
      try
        if ToolServices = nil then
          MessageDlg('ToolServices = nil', mtError, [mbOK], 0)
        else
        begin
          fname := ToolServices.GetCurrentFile;
          if (uppercase(ExtractFileExt(fname)) <> '.PAS') and (uppercase(ExtractFileExt(fname)) <> '.DPR') then
          begin
            MessageDlg('This Expert is for use in .PAS or .DPR files only', mtError, [mbOK], 0);
          end
          else
          begin
            ModIntf := ToolServices.GetModuleInterface(fname);
            if ModIntf = nil then
            begin
              MessageDlg('GetModuleInterface(fname) = nil', mtError, [mbOK], 0);
            end
            else
            try
              EditIntf := Modintf.GetEditorInterface;
              if EditIntf = nil then
              begin
                MessageDlg('Modintf.GetEditorInterface = nil', mtError, [mbOK], 0);
              end
              else
              try
                Screen.Cursor := crHourglass;
                if EditIntf.GetViewCount <= 0 then
                begin
                  MessageDlg('EditIntf.GetViewCount <= 0', mtError, [mbOK], 0);
                end
                else
                begin
                  EditWriter := EditIntf.CreateWriter;
                  if EditWriter = nil then
                  begin
                    MessageDlg('EditIntf.CreateWriter = nil', mtError, [mbOK], 0);
                  end
                  else
                  try

                    EditView := EditIntf.GetView(0);
                    EditPos := EditView.CursorPos;
                    EditView.ConvertPos(true, EditPos, CharPos);
                    lPos := EditView.CharPosToPos(CharPos);
                    EditWriter.CopyTo(lPos);
                    i := EditPos.Col - 1;
                    if Dlg.UseCompilerCond then
                      Command := space(i) + Dlg.CompilerCond + #13#10 + space(i + 2) + Dlg.DebugString + #13#10 +
                      space(i) + '{$ENDIF}' + #13#10
                    else
                      Command := space(i) + Dlg.DebugString + #13#10;
                    EditWriter.Insert(PChar(Command));
                  finally
                    EditWriter.Free;
                    EditView.Free;
                  end;
                end;
              finally
                Screen.Cursor := crDefault;
                EditIntf.Free;
              end
            finally
              ModIntf.Free;
            end
          end
        end;
      except
        on E: Exception do MessageDlg(E.Message, mtError, [mbOK], 0);
      end;
    end;
  finally
    Dlg.Release;
  end;
end;



procedure TOverseerExpert.Execute;
begin
    { nada }
end;

function TOverseerExpert.GetName: string;
begin
  Result := 'Overseer_Expert';
end;

function TOverseerExpert.GetStyle: TExpertStyle;
begin
  Result := esAddIn;
end;

function TOverseerExpert.GetState: TExpertState;
begin
  Result := []
end;

function TOverseerExpert.GetIDString: string;
begin
  Result := 'PKS.OverseerMessageExpert';
end;

procedure Register;
begin
  RegisterLibraryExpert(TOverseerExpert.Create);
end;


end.

